@@ -184,9 +184,10 @@
  	                	polyominoes -root			    \\n\
 - 	                	thornbird -root				    \\n\
  	                	fluidballs -root			    \\n\
+ 	                	anemone -root				    \\n\
   color: 			bubbles -root				    \\n\
-  default-n:			webcollage -root			    \\n\
-  default-n:  \"WebCollage (whacked)\"					      \
+- default-n:			webcollage -root			    \\n\
+- default-n:  \"WebCollage (whacked)\"					      \
 				webcollage -root -filter		      \
 				  'vidwhacker -stdin -stdout'		    \\n\
 - default-n:			vidwhacker -root			    \\n\
@@ -229,6 +230,10 @@
 	   GL:			glforestfire -root			    \\n\
 	   GL:			sballs -root				    \\n\
 	   GL:			cubenetic -root				    \\n\
+	   GL:			spheremonics -root			    \\n\
+	   GL:			lavalite -root				    \\n\
+	   GL:			queens -root				    \\n\
+	   GL:			endgame -root				    \\n\
 									      \
 -				xdaliclock -root -builtin3 -cycle	    \\n\
 - default-n:			xearth -nofork -nostars -ncolors 50	      \
