@@ -175,7 +175,7 @@ static Bool
 grab_keyboard_and_mouse (saver_info *si, Window window, Cursor cursor,
                          int screen_no)
 {
-  Status mstatus, kstatus;
+  Status mstatus = 0, kstatus = 0;
   int i;
   int retries = 4;
 
@@ -1518,7 +1518,7 @@ mouse_screen (saver_info *si)
                              &root_x, &root_y, &win_x, &win_y, &mask))
             {
               if (p->verbose_p)
-                fprintf (stderr, "%s: mouse is on screen %d\n",
+                fprintf (stderr, "%s: mouse is on screen %d of %d\n",
                          blurb(), i, si->nscreens);
               return i;
             }
