@@ -19,7 +19,7 @@
 #endif
 
 #include <GL/gl.h>
-#include <GL/glut.h>
+/*#include <GL/glut.h>*/
 #ifdef HAVE_GLE3
 #include <GL/gle.h>
 #else
