@@ -1,4 +1,4 @@
-/* gltext, Copyright (c) 2001 Jamie Zawinski <jwz@jwz.org>
+/* gltext, Copyright (c) 2001, 2002 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -17,6 +17,8 @@ extern XtAppContext app;
 #define HACK_INIT	init_text
 #define HACK_DRAW	draw_text
 #define HACK_RESHAPE	reshape_text
+#define HACK_HANDLE_EVENT text_handle_event
+#define EVENT_MASK	PointerMotionMask
 #define sws_opts	xlockmore_opts
 
 #define DEF_TEXT        "(default)"
@@ -46,6 +48,10 @@ extern XtAppContext app;
 #include "xlockmore.h"
 #include "colors.h"
 #include "tube.h"
+#include "rotator.h"
+#include "gltrackball.h"
+#include <time.h>
+#include <sys/time.h>
 #include <ctype.h>
 
 #ifdef USE_GL /* whole file */
@@ -63,11 +69,9 @@ extern XtAppContext app;
 
 typedef struct {
   GLXContext *glx_context;
-
-  GLfloat rotx, roty, rotz;	   /* current object rotation */
-  GLfloat dx, dy, dz;		   /* current rotational velocity */
-  GLfloat ddx, ddy, ddz;	   /* current rotational acceleration */
-  GLfloat d_max;		   /* max velocity */
+  rotator *rot;
+  trackball_state *trackball;
+  Bool button_down_p;
 
   GLuint text_list;
 
@@ -75,7 +79,6 @@ typedef struct {
   XColor *colors;
   int ccolor;
 
-  Bool spin_x, spin_y, spin_z;
   char *text;
 
 } text_configuration;
@@ -89,7 +92,7 @@ static Bool do_wander;
 static XrmOptionDescRec opts[] = {
   { "-text",   ".text",   XrmoptionSepArg, 0 },
   { "-spin",   ".spin",   XrmoptionSepArg, 0 },
-  { "+spin",   ".spin",   XrmoptionNoArg, "False" },
+  { "+spin",   ".spin",   XrmoptionNoArg, "" },
   { "-wander", ".wander", XrmoptionNoArg, "True" },
   { "+wander", ".wander", XrmoptionNoArg, "False" }
 };
@@ -114,14 +117,13 @@ reshape_text (ModeInfo *mi, int width, int height)
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
+  gluPerspective (30.0, 1/h, 1.0, 100.0);
 
-  gluPerspective( 30.0, 1/h, 1.0, 100.0 );
-  gluLookAt( 0.0, 0.0, 15.0,
-             0.0, 0.0, 0.0,
-             0.0, 1.0, 0.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-  glTranslatef(0.0, 0.0, -15.0);
+  gluLookAt( 0.0, 0.0, 30.0,
+             0.0, 0.0, 0.0,
+             0.0, 1.0, 0.0);
 
   glClear(GL_COLOR_BUFFER_BIT);
 }
@@ -150,77 +152,6 @@ gl_init (ModeInfo *mi)
 }
 
 
-/* lifted from lament.c */
-#define RAND(n) ((long) ((random() & 0x7fffffff) % ((long) (n))))
-#define RANDSIGN() ((random() & 1) ? 1 : -1)
-
-static void
-rotate(GLfloat *pos, GLfloat *v, GLfloat *dv, GLfloat max_v)
-{
-  double ppos = *pos;
-
-  /* tick position */
-  if (ppos < 0)
-    ppos = -(ppos + *v);
-  else
-    ppos += *v;
-
-  if (ppos > 1.0)
-    ppos -= 1.0;
-  else if (ppos < 0)
-    ppos += 1.0;
-
-  if (ppos < 0) abort();
-  if (ppos > 1.0) abort();
-  *pos = (*pos > 0 ? ppos : -ppos);
-
-  /* accelerate */
-  *v += *dv;
-
-  /* clamp velocity */
-  if (*v > max_v || *v < -max_v)
-    {
-      *dv = -*dv;
-    }
-  /* If it stops, start it going in the other direction. */
-  else if (*v < 0)
-    {
-      if (random() % 4)
-	{
-	  *v = 0;
-
-	  /* keep going in the same direction */
-	  if (random() % 2)
-	    *dv = 0;
-	  else if (*dv < 0)
-	    *dv = -*dv;
-	}
-      else
-	{
-	  /* reverse gears */
-	  *v = -*v;
-	  *dv = -*dv;
-	  *pos = -*pos;
-	}
-    }
-
-  /* Alter direction of rotational acceleration randomly. */
-  if (! (random() % 120))
-    *dv = -*dv;
-
-  /* Change acceleration very occasionally. */
-  if (! (random() % 200))
-    {
-      if (*dv == 0)
-	*dv = 0.00001;
-      else if (random() & 1)
-	*dv *= 1.2;
-      else
-	*dv *= 0.8;
-    }
-}
-
-
 static void
 parse_text (ModeInfo *mi)
 {
@@ -280,10 +211,44 @@ parse_text (ModeInfo *mi)
 }
 
 
+Bool
+text_handle_event (ModeInfo *mi, XEvent *event)
+{
+  text_configuration *tp = &tps[MI_SCREEN(mi)];
+
+  if (event->xany.type == ButtonPress &&
+      event->xbutton.button & Button1)
+    {
+      tp->button_down_p = True;
+      gltrackball_start (tp->trackball,
+                         event->xbutton.x, event->xbutton.y,
+                         MI_WIDTH (mi), MI_HEIGHT (mi));
+      return True;
+    }
+  else if (event->xany.type == ButtonRelease &&
+           event->xbutton.button & Button1)
+    {
+      tp->button_down_p = False;
+      return True;
+    }
+  else if (event->xany.type == MotionNotify &&
+           tp->button_down_p)
+    {
+      gltrackball_track (tp->trackball,
+                         event->xmotion.x, event->xmotion.y,
+                         MI_WIDTH (mi), MI_HEIGHT (mi));
+      return True;
+    }
+
+  return False;
+}
+
+
 void 
 init_text (ModeInfo *mi)
 {
   text_configuration *tp;
+  int i;
 
   if (!tps) {
     tps = (text_configuration *)
@@ -303,37 +268,18 @@ init_text (ModeInfo *mi)
     reshape_text (mi, MI_WIDTH(mi), MI_HEIGHT(mi));
   }
 
-  tp->rotx = frand(1.0) * RANDSIGN();
-  tp->roty = frand(1.0) * RANDSIGN();
-  tp->rotz = frand(1.0) * RANDSIGN();
-
-  /* bell curve from 0-6 degrees, avg 3 */
-  tp->dx = (frand(1) + frand(1) + frand(1)) / (360/2);
-  tp->dy = (frand(1) + frand(1) + frand(1)) / (360/2);
-  tp->dz = (frand(1) + frand(1) + frand(1)) / (360/2);
-
-  tp->d_max = tp->dx * 2;
-
-  tp->ddx = 0.00006 + frand(0.00003);
-  tp->ddy = 0.00006 + frand(0.00003);
-  tp->ddz = 0.00006 + frand(0.00003);
-
-
-  tp->ncolors = 255;
-  tp->colors = (XColor *) calloc(tp->ncolors, sizeof(XColor));
-  make_smooth_colormap (0, 0, 0,
-                        tp->colors, &tp->ncolors,
-                        False, 0, False);
-
-  parse_text (mi);
-
   {
+    Bool spinx=False, spiny=False, spinz=False;
+    double spin_speed   = 1.0;
+    double wander_speed = 0.05;
+    double spin_accel   = 1.0;
+
     char *s = do_spin;
     while (*s)
       {
-        if      (*s == 'x' || *s == 'X') tp->spin_x = 1;
-        else if (*s == 'y' || *s == 'Y') tp->spin_y = 1;
-        else if (*s == 'z' || *s == 'Z') tp->spin_z = 1;
+        if      (*s == 'x' || *s == 'X') spinx = True;
+        else if (*s == 'y' || *s == 'Y') spiny = True;
+        else if (*s == 'z' || *s == 'Z') spinz = True;
         else
           {
             fprintf (stderr,
@@ -343,8 +289,32 @@ init_text (ModeInfo *mi)
           }
         s++;
       }
+
+    tp->rot = make_rotator (spinx ? spin_speed : 0,
+                            spiny ? spin_speed : 0,
+                            spinz ? spin_speed : 0,
+                            spin_accel,
+                            do_wander ? wander_speed : 0,
+                            False);
+    tp->trackball = gltrackball_init ();
   }
 
+  tp->ncolors = 255;
+  tp->colors = (XColor *) calloc(tp->ncolors, sizeof(XColor));
+  make_smooth_colormap (0, 0, 0,
+                        tp->colors, &tp->ncolors,
+                        False, 0, False);
+
+  /* brighter colors, please... */
+  for (i = 0; i < tp->ncolors; i++)
+    {
+      tp->colors[i].red   = (tp->colors[i].red   / 2) + 32767;
+      tp->colors[i].green = (tp->colors[i].green / 2) + 32767;
+      tp->colors[i].blue  = (tp->colors[i].blue  / 2) + 32767;
+    }
+
+  parse_text (mi);
+
 }
 
 
@@ -366,7 +336,7 @@ fill_character (GLUTstrokeFont font, int c, Bool wire)
   ch = &(fontinfo->ch[c]);
   if (ch)
     {
-      GLfloat lx, ly;
+      GLfloat lx=0, ly=0;
       for (i = ch->num_strokes, stroke = ch->stroke;
            i > 0; i--, stroke++) {
         for (j = stroke->num_coords, coord = stroke->coord;
@@ -518,41 +488,21 @@ draw_text (ModeInfo *mi)
   glScalef(1.1, 1.1, 1.1);
 
   {
-    GLfloat x, y, z;
+    double x, y, z;
+    get_position (tp->rot, &x, &y, &z, !tp->button_down_p);
+    glTranslatef((x - 0.5) * 8,
+                 (y - 0.5) * 8,
+                 (z - 0.5) * 8);
 
-    if (do_wander)
-      {
-        static int frame = 0;
+    gltrackball_rotate (tp->trackball);
 
-#       define SINOID(SCALE,SIZE) \
-        ((((1 + sin((frame * (SCALE)) / 2 * M_PI)) / 2.0) * (SIZE)) - (SIZE)/2)
-
-        x = SINOID(0.031, 9.0);
-        y = SINOID(0.023, 9.0);
-        z = SINOID(0.017, 9.0);
-        frame++;
-        glTranslatef(x, y, z);
-      }
-
-    if (tp->spin_x || tp->spin_y || tp->spin_z)
-      {
-        x = tp->rotx;
-        y = tp->roty;
-        z = tp->rotz;
-        if (x < 0) x = 1 - (x + 1);
-        if (y < 0) y = 1 - (y + 1);
-        if (z < 0) z = 1 - (z + 1);
-
-        if (tp->spin_x) glRotatef(x * 360, 1.0, 0.0, 0.0);
-        if (tp->spin_y) glRotatef(y * 360, 0.0, 1.0, 0.0);
-        if (tp->spin_z) glRotatef(z * 360, 0.0, 0.0, 1.0);
-
-        rotate(&tp->rotx, &tp->dx, &tp->ddx, tp->d_max);
-        rotate(&tp->roty, &tp->dy, &tp->ddy, tp->d_max);
-        rotate(&tp->rotz, &tp->dz, &tp->ddz, tp->d_max);
-      }
+    get_rotation (tp->rot, &x, &y, &z, !tp->button_down_p);
+    glRotatef (x * 360, 1.0, 0.0, 0.0);
+    glRotatef (y * 360, 0.0, 1.0, 0.0);
+    glRotatef (z * 360, 0.0, 0.0, 1.0);
   }
 
+
   glColor4fv (white);
 
   color[0] = tp->colors[tp->ccolor].red   / 65536.0;
